use chrono::offset::Utc;
use rusqlite::{types::ToSqlOutput, ToSql};
use std::collections::HashMap;
use tokio::sync::Mutex;
use tauri::{Manager, State};

use crate::types::*;

pub mod app_state;
pub mod data_door;

use app_state::AppState;

fn parse_inventory(data: HashMap<String, String>) -> Result<Inventory, ()> {
	let a = data.get("cafe-inventory-acc").ok_or(())?;
	let acc: Account = Account::try_from(a.as_ref())?;
	let mut vouchers = Vec::new();
	for s in ["aldi", "dm", "lidl", "rewe", "tegut"] {
		let Ok(store) = s.try_into() else {
			println!("Did not find '{}' in inventory data.", s);
			continue;
		};
		match data.get(&format!("cafe-inventory-{}", s)) {
			None => (),
			Some(c) => {
				let c = if c == "" {"0"} else {c};
				let Ok(count) = c.parse() else {
					println!("Invalid count '{}' for '{}' in inventory data.", c, s);
					continue;
				};
				let v = VoucherInventory { store, count };
				vouchers.push(v);
			},
		}
	}
	unimplemented!()
}

#[tauri::command]
pub async fn inventory(
	data: HashMap<String, String>,
	state: State<'_, Mutex<AppState>>,
) -> Result<(), ()> {
	let now = Utc::now().timestamp();
	let state = state.lock().await;
	let inv = parse_inventory(data)?;
	for v in inv.vouchers {
		state.db.execute(
			"INSERT INTO voucher_inventory VALUES ()",
			(
				inv.acc,
				v.store,
				v.count,
				now,
			),
		)
		.map_err(|e| println!("{:?}", e))?;
	};
	Ok(())
}

#[tauri::command]
pub async fn swap(
	store: &str,
	acc: i64,
	state: State<'_, Mutex<AppState>>,
) -> Result<(), ()> {
	let state = state.lock().await;
	let store: Store = store.try_into()?;
	state.db.execute(
		"INSERT INTO swap VALUES (?1, ?2, ?3, ?4, ?5)",
		(
			store,
			acc,
			i64::from_ne_bytes(state.id.to_ne_bytes()),
			Utc::now().timestamp(),
			false,
		),
	)
	.map_err(|e| println!("{:?}", e))?;
	Ok(())
}

#[tauri::command]
pub async fn count(state: State<'_, Mutex<AppState>>) -> Result<String, ()> {
	let state = state.lock().await;
	let mut stmt =
		state.db.prepare("SELECT COUNT(*) FROM swap")
			.map_err(|e| println!("{:?}", e))?;
	let mut rows = stmt.query([]).map_err(|e| println!("{:?}", e))?;
	let row = rows.next().map_err(|e| println!("{:?}", e))?;
	let row = match row {
		Some(r) => Ok(r),
		None => {
			println!("No rows");
			Err(())
		}
	}?;
	let cnt: u64 = row.get(0).map_err(|e| println!("{:?}", e))?;
	Ok(cnt.to_string())
}
